searchData={"content_type":"text/markdown","items":[{"doc":"# LoggerExporter\n\nExport your logs to the service of your choice.\n\nI created this library because I wanted to export logs to a different service in\nHeroku. There is no simple way to export your logs.","ref":"LoggerExporter.html","title":"LoggerExporter","type":"module"},{"doc":"- Loki\n- Mezmo (LogDNA)\n\nImplement your own exporter using `LoggerExporter.Exporters.Exporter` behaviour.","ref":"LoggerExporter.html#module-supported-exporters","title":"Supported exporters: - LoggerExporter","type":"module"},{"doc":"- Basic. By default, it will read your logger `:console` format configuration.\n  Example:\n  ```elixir\n  config :logger, :console, format: \"$time $message\"\n  ```\n  Defaults to `$time $metadata[$level] $message`\n\nImplement your own formatter using `LoggerExporter.Formatters.Formatter` behaviour.","ref":"LoggerExporter.html#module-supported-formatters","title":"Supported formatters: - LoggerExporter","type":"module"},{"doc":"A plug for logging request information. It will log the method, path, params,\nstatus and duration.\n\nYou can add it to your MyApp.Endpoint:\n```elixir\n  # Log level defaults to :info\n  plug LoggerExporter.Loggers.Plug\n\n  # Dynamic log\n  plug LoggerExporter.Loggers.Plug, log: {__MODULE__, :log_level, []}\n\n  # Disables logging for routes like /health_check\n  def log_level(%{path_info: [\"health_check\"]}), do: false\n  def log_level(_), do: :info\n```","ref":"LoggerExporter.html#module-plug-logger","title":"Plug Logger - LoggerExporter","type":"module"},{"doc":"Add `logger_exporter` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:logger_exporter, \"~> 0.4.0\"}\n  ]\nend\n```","ref":"LoggerExporter.html#module-installation","title":"Installation - LoggerExporter","type":"module"},{"doc":"By default, the timestamp sent for each log to the external service is in utc: `System.os_time(:nanosecond)`\n\n| option           | description                                                                                                   | default                                    |\n|------------------|---------------------------------------------------------------------------------------------------------------|--------------------------------------------|\n| level            | The logger level to report.                                                                                   | `:info`                                    |\n| formatter        | Allows the selection of a formatter implementation.                                                           | `LoggerExporter.Formatters.BasicFormatter` |\n| metadata         | Metadata to log.                                                                                              | `[]`                                       |\n| exporter         | Allows selection of a exporter implementation.                                                                | `LoggerExporter.Exporters.LokiExporter`    |\n| host             | The host of the service without the path. The path is inferred by the exporter.                               | No default. Required                       |\n| app_name         | The name of the app to use as label.                                                                          | No default. Required                       |\n| environment_name | The name of the app to use as label.                                                                          | No default. Required                       |\n| send_to_http     | If set to false, the library will not make any actual API calls. This is useful for test or dev environments. | `true`                                       |\n| http_authh       | See below for configuration                                                                                   | No default                                 |\n\n\n#","ref":"LoggerExporter.html#module-configuration","title":"Configuration - LoggerExporter","type":"module"},{"doc":"Supported authentication methods:\n- Basic\n- Bearer\n- Custom\n\n| auth     | examples                                   | result                                           |\n| -------- | ------------------------------------------ | ------------------------------------------------ |\n| basic    | `{:basic, \"user\", \"pxIsldPlwty\"}`          | `\"Authorization: Basic CshL2XlkX57cww==\"`        |\n| bearer   | `{:bearer, \"WmRUuBOnTjDwP6jo3bno\"}`        | `\"Authorization: Bearer WmRUuBOnTjDwP6jo3bno\"`   |\n| custom   | `{:header, \"apiKey\", \"dAWRQQkZCc2A==\"}`    | `\"apiKey: dAWRQQkZCc2A==\"`                       |","ref":"LoggerExporter.html#module-htpp-auth","title":"HTPP Auth - LoggerExporter","type":"module"},{"doc":"1.  Add the following to deps section of your mix.exs: `{:logger_exporter, \"~> 0.4.0\"}`\n    and then `mix deps.get`\n\n2.  Add `LoggerExporter.Backend` to your logger's backends configuration\n\n    ```\n    config :logger,\n      backends: [:console, LoggerExporter.Backend]\n    ```\n\n3.  Add config related to the exporter and other fields.\n    ie. for `LokiExporter`\n\n    ```elixir\n    config :logger, LoggerExporter,\n      host: \"http://localhost:3100\",\n      app_name: \"my_app\",\n      environment_name: config_env(),\n      http_auth: {:basic, System.fetch_env!(\"LOKI_USER\"), System.fetch_env!(\"LOKI_PASSWORD\")},\n      metadata: [:request_id]\n    ```\n\n4.  Start the LoggerExporter GenServer in the supervised children list.\n    In `application.ex` add to the children list:\n\n    ```elixir\n    children [\n      ...\n      LoggerExporter\n    ]\n    ```\n5. (Optional) Add custom Plug logger.\n  In `MyApp.Endpoint` add the plug after `Plug.Parsers`.\n  If you see duplicate logs, remove `Plug.Telemetry` from your endpoint.\n\n    ```elixir\n    plug Plug.Parsers,\n      ...\n\n    plug LoggerExporter.Loggers.Plug\n    ```","ref":"LoggerExporter.html#module-usage-in-phoenix","title":"Usage in Phoenix - LoggerExporter","type":"module"},{"doc":"If you want to log in JSON format, you can use the formatter of another library:\n[logger_json](https://github.com/Nebo15/logger_json)\n\nYou need to implement the following formatter using the `LoggerExporter.Formatters.Formatter` behaviour.\n\n```elixir\ndefmodule MyFormatter do\n  @behaviour LoggerExporter.Formatters.Formatter\n\n  alias LoggerJSON.Formatters.BasicLogger\n\n  @impl true\n  def format_event(level, msg, ts, md, md_keys) do\n    BasicLogger.format_event(level, msg, ts, md, md_keys, [])\n    |> Jason.encode!()\n  end\nend\n```\n\nThen, configure it like this:\n```elixir\nconfig :logger, LoggerExporter,\n  formatter: MyFormatter\n```\n\nTada! You have JSON logs!","ref":"LoggerExporter.html#module-json-formatter-example","title":"JSON Formatter Example - LoggerExporter","type":"module"},{"doc":"Telemetry integration for logging and error reporting.\n\nThere is a default logger for you to attach. It logs when the `status` is `:error`\n\nIn your `application.ex`\n\n```elixir\n:ok = LoggerExporter.Telemetry.attach_default_logger()\n```\n\n#","ref":"LoggerExporter.html#module-telemetry","title":"Telemetry - LoggerExporter","type":"module"},{"doc":"LoggerExporter emits the following events for processing each batch (sending it through http)\n\n- `[:logger_exporter, :batch, :start]` -- starting to process the events\n- `[:logger_exporter, :batch, :stop]` -- after the events is processed\n- `[:logger_exporter, :batch, :exception]` -- after the events are processed\n\nThe following chart shows which metadata you can expect for each event:\n\n| event        | measures       | metadata                               |\n| ------------ | -------------- | ---------                              |\n| `:start`     | `:system_time` | `:events`                              |\n| `:stop`      | `:duration`    | `:events, :status, :response`          |\n| `:exception` | `:duration`    | `:events, :kind, :reason, :stacktrace` |\n\nMetadata\n* `:events` - the list of `LoggerExporter.Event` processed\n* `:status` - either `:ok` or `:error`\n* `:response` - information of the response. Is a `Finch.Response` struct","ref":"LoggerExporter.html#module-http-post-batch-events","title":"HTTP Post batch events - LoggerExporter","type":"module"},{"doc":"","ref":"LoggerExporter.html#child_spec/1","title":"LoggerExporter.child_spec/1","type":"function"},{"doc":"","ref":"LoggerExporter.html#start_link/1","title":"LoggerExporter.start_link/1","type":"function"},{"doc":"Take specified keys from the metadata","ref":"LoggerExporter.html#take_metadata/2","title":"LoggerExporter.take_metadata/2","type":"function"},{"doc":"LoggerExporter Backend.\n\nUsed when configuring the `Logger` :backends","ref":"LoggerExporter.Backend.html","title":"LoggerExporter.Backend","type":"module"},{"doc":"GenServer that batches the events in a `:queue`\n\nSends the batch to the HTTPClient","ref":"LoggerExporter.Batcher.html","title":"LoggerExporter.Batcher","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"LoggerExporter.Batcher.html#child_spec/1","title":"LoggerExporter.Batcher.child_spec/1","type":"function"},{"doc":"Enqueue events to the Batcher\nEvents will be processed and exported to the service","ref":"LoggerExporter.Batcher.html#enqueue/1","title":"LoggerExporter.Batcher.enqueue/1","type":"function"},{"doc":"","ref":"LoggerExporter.Batcher.html#get_queue/0","title":"LoggerExporter.Batcher.get_queue/0","type":"function"},{"doc":"","ref":"LoggerExporter.Batcher.html#start_link/1","title":"LoggerExporter.Batcher.start_link/1","type":"function"},{"doc":"Event struct that holds the formatted log","ref":"LoggerExporter.Event.html","title":"LoggerExporter.Event","type":"module"},{"doc":"","ref":"LoggerExporter.Event.html#t:t/0","title":"LoggerExporter.Event.t/0","type":"type"},{"doc":"Behaviour that should be implemented by exporters.\nExample implementation in `LoggerExporter.Exporters.LokiExporter`","ref":"LoggerExporter.Exporters.Exporter.html","title":"LoggerExporter.Exporters.Exporter","type":"behaviour"},{"doc":"Body to sent to the external service\n\nThe body will be encoded by the json library","ref":"LoggerExporter.Exporters.Exporter.html#c:body/1","title":"LoggerExporter.Exporters.Exporter.body/1","type":"callback"},{"doc":"Headers\n\n[{\"Content-Type\", \"application/json\"}] is added by default","ref":"LoggerExporter.Exporters.Exporter.html#c:headers/0","title":"LoggerExporter.Exporters.Exporter.headers/0","type":"callback"},{"doc":"LokiExporter\n\nTransforms the events in a struct that Loki understands","ref":"LoggerExporter.Exporters.LokiExporter.html","title":"LoggerExporter.Exporters.LokiExporter","type":"module"},{"doc":"MezmoExporter","ref":"LoggerExporter.Exporters.MezmoExporter.html","title":"LoggerExporter.Exporters.MezmoExporter","type":"module"},{"doc":"Basic formatter.\n\nIf there is no `:logger -> :console -> :format`, it sets the format to: \"$time $metadata[$level] $message\"","ref":"LoggerExporter.Formatters.BasicFormatter.html","title":"LoggerExporter.Formatters.BasicFormatter","type":"module"},{"doc":"Behaviour that should be implemented by log formatters.\nExample implementation in `LoggerExporter.Formatters.BasicLogger`","ref":"LoggerExporter.Formatters.Formatter.html","title":"LoggerExporter.Formatters.Formatter","type":"behaviour"},{"doc":"Format event callback.","ref":"LoggerExporter.Formatters.Formatter.html#c:format_event/5","title":"LoggerExporter.Formatters.Formatter.format_event/5","type":"callback"},{"doc":"A plug for logging request information in the format:\n\n    method=POST path=/users params=%{\"user\" => %{\"name\" => \"Juan\"}} status=201 duration=101ms\n\nTo use it, just plug it into the desired module.\n\n    plug LoggerExporter.Loggers.Plug,\n      log: :debug,\n      filter_parameters_fn: &Phoenix.Logger.filter_values/1","ref":"LoggerExporter.Loggers.Plug.html","title":"LoggerExporter.Loggers.Plug","type":"module"},{"doc":"* `:log` - The log level at which this plug should log its request information.\n    Default is `:info`.\n      * Configure log level. The [list of supported levels](https://hexdocs.pm/logger/Logger.html#module-levels)\n    is available in the `Logger` documentation.\n      * Configure log level dynamically: `plug LoggerExporter.Logger.Plug, log: {Mod, Fun, Args}`\n  * `:filter_parameters_fn` - The Function to call with conn.params. Filter\n    params that doesn't need to be logged i.e. {\"password\": \"[FILTERED]\"}.","ref":"LoggerExporter.Loggers.Plug.html#module-options","title":"Options - LoggerExporter.Loggers.Plug","type":"module"},{"doc":"In some cases you may wish to set the log level dynamically\n  on a per-request basis. To do so, set the `:log` option to\n  a tuple, `{Mod, Fun, Args}`. The `Plug.Conn.t()` for the\n  request will be prepended to the provided list of arguments.\n\n  When invoked, your function must return a\n  [`Logger.level()`](`t:Logger.level()/0`) or `false` to\n  disable logging for the request.\n\n  For example, in your Endpoint you might do something like this:\n\n        # lib/my_app_web/endpoint.ex\n        plug LoggerExporter.Loggers.Plug,\n          log: {__MODULE__, :log_level, []}\n\n        # Disables logging for routes like /health_check\n        def log_level(%{path_info: [\"health_check\"]}), do: false\n        def log_level(_), do: :info","ref":"LoggerExporter.Loggers.Plug.html#module-dynamic-log-level","title":"Dynamic log level - LoggerExporter.Loggers.Plug","type":"module"},{"doc":"Telemetry default handler for [:start, :stop, :exception] batch events","ref":"LoggerExporter.Telemetry.html","title":"LoggerExporter.Telemetry","type":"module"},{"doc":"","ref":"LoggerExporter.Telemetry.html#attach_default_logger/0","title":"LoggerExporter.Telemetry.attach_default_logger/0","type":"function"},{"doc":"","ref":"LoggerExporter.Telemetry.html#handle_event/4","title":"LoggerExporter.Telemetry.handle_event/4","type":"function"}]}